library benchmarks.src.compiler.compiler_benchmark;

import "package:angular2/src/dom/browser_adapter.dart" show BrowserDomAdapter;
import "package:angular2/src/facade/async.dart" show PromiseWrapper;
import "package:angular2/src/facade/collection.dart"
    show List, ListWrapper, Map, MapWrapper;
import "package:angular2/src/facade/lang.dart" show DateWrapper, Type, print;
import "package:angular2/src/render/dom/shadow_dom/native_shadow_dom_strategy.dart"
    show NativeShadowDomStrategy;
import "package:angular2/change_detection.dart"
    show Parser, Lexer, DynamicChangeDetection;
import "package:angular2/src/core/compiler/compiler.dart"
    show Compiler, CompilerCache;
import "package:angular2/src/core/compiler/directive_resolver.dart"
    show DirectiveResolver;
import "package:angular2/src/core/annotations_impl/view.dart" as viewModule;
import "package:angular2/angular2.dart" show Component, Directive, View;
import "package:angular2/src/render/dom/compiler/view_loader.dart"
    show ViewLoader;
import "package:angular2/src/core/compiler/view_resolver.dart"
    show ViewResolver;
import "package:angular2/src/services/url_resolver.dart" show UrlResolver;
import "package:angular2/src/services/app_root_url.dart" show AppRootUrl;
import "package:angular2/src/core/compiler/component_url_mapper.dart"
    show ComponentUrlMapper;
import "package:angular2/src/reflection/reflection.dart" show reflector;
import "package:angular2/src/reflection/reflection_capabilities.dart"
    show ReflectionCapabilities;
import "package:angular2/src/test_lib/benchmark_util.dart"
    show getIntParameter, bindAction;
import "package:angular2/src/core/compiler/proto_view_factory.dart"
    show ProtoViewFactory;
import "package:angular2/src/render/dom/compiler/compiler.dart" as rc;

main() {
  BrowserDomAdapter.makeCurrent();
  var count = getIntParameter("elements");
  reflector.reflectionCapabilities = new ReflectionCapabilities();
  var reader = new DirectiveResolver();
  var cache = new CompilerCache();
  var viewResolver = new MultipleViewResolver(
      count, [BenchmarkComponentNoBindings, BenchmarkComponentWithBindings]);
  var urlResolver = new UrlResolver();
  var shadowDomStrategy = new NativeShadowDomStrategy();
  var renderCompiler = new rc.DefaultDomCompiler(new Parser(new Lexer()),
      shadowDomStrategy, new ViewLoader(null, null, null));
  var compiler = new Compiler(reader, cache, viewResolver,
      new ComponentUrlMapper(), urlResolver, renderCompiler,
      new ProtoViewFactory(new DynamicChangeDetection(null)),
      new FakeAppRootUrl());
  measureWrapper(func, desc) {
    return () {
      var begin = DateWrapper.now();
      print('''[${ desc}] Begin...''');
      var onSuccess = (_) {
        var elapsedMs = DateWrapper.toMillis(DateWrapper.now()) -
            DateWrapper.toMillis(begin);
        print('''[${ desc}] ...done, took ${ elapsedMs} ms''');
      };
      PromiseWrapper.then(func(), onSuccess, null);
    };
  }
  compileNoBindings() {
    cache.clear();
    return compiler.compileInHost(BenchmarkComponentNoBindings);
  }
  compileWithBindings() {
    cache.clear();
    return compiler.compileInHost(BenchmarkComponentWithBindings);
  }
  bindAction(
      "#compileNoBindings", measureWrapper(compileNoBindings, "No Bindings"));
  bindAction("#compileWithBindings",
      measureWrapper(compileWithBindings, "With Bindings"));
}
@Directive(selector: "[dir0]", properties: const ["prop: attr0"])
class Dir0 {}
@Directive(selector: "[dir1]", properties: const ["prop: attr1"])
class Dir1 {
  Dir1(Dir0 dir0) {}
}
@Directive(selector: "[dir2]", properties: const ["prop: attr2"])
class Dir2 {
  Dir2(Dir1 dir1) {}
}
@Directive(selector: "[dir3]", properties: const ["prop: attr3"])
class Dir3 {
  Dir3(Dir2 dir2) {}
}
@Directive(selector: "[dir4]", properties: const ["prop: attr4"])
class Dir4 {
  Dir4(Dir3 dir3) {}
}
class MultipleViewResolver extends ViewResolver {
  num _multiple;
  Map<dynamic, dynamic> _cache;
  MultipleViewResolver(num multiple, List<Type> components) : super() {
    /* super call moved to initializer */;
    this._multiple = multiple;
    this._cache = new Map();
    ListWrapper.forEach(components, (c) => this._warmUp(c));
  }
  _warmUp(Type component) {
    var view = super.resolve(component);
    var multiplier = ListWrapper.createFixedSize(this._multiple);
    for (var i = 0; i < this._multiple; ++i) {
      multiplier[i] = view.template;
    }
    this._cache[component] = ListWrapper.join(multiplier, "");
  }
  viewModule.View resolve(Type component) {
    var view = super.resolve(component);
    var myView = new viewModule.View(
        template: (this._cache[component] as String),
        directives: view.directives);
    return myView;
  }
}
@Component(selector: "cmp-nobind")
@View(directives: const [Dir0, Dir1, Dir2, Dir3, Dir4], template: '''
<div class="class0 class1 class2 class3 class4 " nodir0="" attr0="value0" nodir1="" attr1="value1" nodir2="" attr2="value2" nodir3="" attr3="value3" nodir4="" attr4="value4">
  <div class="class0 class1 class2 class3 class4 " nodir0="" attr0="value0" nodir1="" attr1="value1" nodir2="" attr2="value2" nodir3="" attr3="value3" nodir4="" attr4="value4">
    <div class="class0 class1 class2 class3 class4 " nodir0="" attr0="value0" nodir1="" attr1="value1" nodir2="" attr2="value2" nodir3="" attr3="value3" nodir4="" attr4="value4">
      <div class="class0 class1 class2 class3 class4 " nodir0="" attr0="value0" nodir1="" attr1="value1" nodir2="" attr2="value2" nodir3="" attr3="value3" nodir4="" attr4="value4">
        <div class="class0 class1 class2 class3 class4 " nodir0="" attr0="value0" nodir1="" attr1="value1" nodir2="" attr2="value2" nodir3="" attr3="value3" nodir4="" attr4="value4">
        </div>
      </div>
    </div>
  </div>
</div>''')
class BenchmarkComponentNoBindings {}
@Component(selector: "cmp-withbind")
@View(directives: const [Dir0, Dir1, Dir2, Dir3, Dir4], template: '''
<div class="class0 class1 class2 class3 class4 " dir0="" [attr0]="value0" dir1="" [attr1]="value1" dir2="" [attr2]="value2" dir3="" [attr3]="value3" dir4="" [attr4]="value4">
  {{inter0}}{{inter1}}{{inter2}}{{inter3}}{{inter4}}
  <div class="class0 class1 class2 class3 class4 " dir0="" [attr0]="value0" dir1="" [attr1]="value1" dir2="" [attr2]="value2" dir3="" [attr3]="value3" dir4="" [attr4]="value4">
    {{inter0}}{{inter1}}{{inter2}}{{inter3}}{{inter4}}
    <div class="class0 class1 class2 class3 class4 " dir0="" [attr0]="value0" dir1="" [attr1]="value1" dir2="" [attr2]="value2" dir3="" [attr3]="value3" dir4="" [attr4]="value4">
      {{inter0}}{{inter1}}{{inter2}}{{inter3}}{{inter4}}
      <div class="class0 class1 class2 class3 class4 " dir0="" [attr0]="value0" dir1="" [attr1]="value1" dir2="" [attr2]="value2" dir3="" [attr3]="value3" dir4="" [attr4]="value4">
        {{inter0}}{{inter1}}{{inter2}}{{inter3}}{{inter4}}
        <div class="class0 class1 class2 class3 class4 " dir0="" [attr0]="value0" dir1="" [attr1]="value1" dir2="" [attr2]="value2" dir3="" [attr3]="value3" dir4="" [attr4]="value4">
          {{inter0}}{{inter1}}{{inter2}}{{inter3}}{{inter4}}
        </div>
      </div>
    </div>
  </div>
</div>''')
class BenchmarkComponentWithBindings {}
class FakeAppRootUrl extends AppRootUrl {
  get value {
    return "";
  }
}
