library benchmarks.src.costs.index;

import "package:angular2/angular2.dart"
    show
        bootstrap,
        Component,
        Directive,
        DynamicComponentLoader,
        ElementRef,
        View;
import "package:angular2/src/core/life_cycle/life_cycle.dart" show LifeCycle;
import "package:angular2/src/facade/collection.dart" show List, ListWrapper;
import "package:angular2/src/reflection/reflection.dart" show reflector;
import "package:angular2/src/reflection/reflection_capabilities.dart"
    show ReflectionCapabilities;
import "package:angular2/src/test_lib/benchmark_util.dart"
    show getIntParameter, bindAction;
import "package:angular2/directives.dart" show NgIf, NgFor;

var testList = null;
main() {
  reflector.reflectionCapabilities = new ReflectionCapabilities();
  var size = getIntParameter("size");
  testList = ListWrapper.createFixedSize(size);
  bootstrap(AppComponent).then((ref) {
    var injector = ref.injector;
    AppComponent app = injector.get(AppComponent);
    var lifeCycle = injector.get(LifeCycle);
    bindAction("#reset", () {
      app.reset();
      lifeCycle.tick();
    });
    // Baseline (plain components)
    bindAction("#createPlainComponents", () {
      app.createPlainComponents();
      lifeCycle.tick();
    });
    // Components with decorators
    bindAction("#createComponentsWithDirectives", () {
      app.createComponentsWithDirectives();
      lifeCycle.tick();
    });
    // Components with decorators
    bindAction("#createDynamicComponents", () {
      app.createDynamicComponents();
      lifeCycle.tick();
    });
  });
}
@Component(selector: "dummy")
@View(template: '''<div></div>''')
class DummyComponent {}
@Directive(selector: "[dummy-decorator]")
class DummyDirective {}
@Directive(selector: "dynamic-dummy")
class DynamicDummy {
  DynamicDummy(DynamicComponentLoader loader, ElementRef location) {
    loader.loadNextToLocation(DummyComponent, location);
  }
}
@Component(selector: "app")
@View(
    directives: const [
  NgIf,
  NgFor,
  DummyComponent,
  DummyDirective,
  DynamicDummy
],
    template: '''
    <div *ng-if="testingPlainComponents">
      <dummy *ng-for="#i of list"></dummy>
    </div>

    <div *ng-if="testingWithDirectives">
      <dummy dummy-decorator *ng-for="#i of list"></dummy>
    </div>

    <div *ng-if="testingDynamicComponents">
      <dynamic-dummy *ng-for="#i of list"></dynamic-dummy>
    </div>
  ''')
class AppComponent {
  List<dynamic> list;
  bool testingPlainComponents;
  bool testingWithDirectives;
  bool testingDynamicComponents;
  AppComponent() {
    this.reset();
  }
  void reset() {
    this.list = [];
    this.testingPlainComponents = false;
    this.testingWithDirectives = false;
    this.testingDynamicComponents = false;
  }
  void createPlainComponents() {
    this.list = testList;
    this.testingPlainComponents = true;
  }
  void createComponentsWithDirectives() {
    this.list = testList;
    this.testingWithDirectives = true;
  }
  void createDynamicComponents() {
    this.list = testList;
    this.testingDynamicComponents = true;
  }
}
